/* SupplierSpliterator.vala
 *
 * Copyright (C) 2018  kosmolot (kosmolot17@yandex.com)
 *
 * This file is part of Gpseq.
 *
 * Gpseq is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * Gpseq is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Gpseq.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Gpseq {
	/**
	 * An infinite unordered spliterator, which each element is generated by a
	 * supplier.
	 */
	public class SupplierSpliterator<G> : Object, Spliterator<G> {
		private Supplier<G> _supplier;

		/**
		 * Creates a new supplier spliterator.
		 * @param supplier a supplier
		 */
		public SupplierSpliterator (Supplier<G> supplier) {
			_supplier = supplier;
		}

		public Spliterator<G>? try_split () {
			return new SupplierSpliterator<G>(_supplier);
		}

		public bool try_advance (Func<G> consumer) {
			consumer(_supplier.supply());
			return true;
		}

		public int estimated_size {
			get {
				return -1;
			}
		}

		public bool is_size_known {
			get {
				return true;
			}
		}
	}
}
